package com.uoc.taller.adapter.encryption;

import com.uoc.taller.domain.entity.exceptions.EncryptionException;
import com.uoc.taller.domain.port.EncryptionService;

import javax.crypto.SecretKeyFactory;
import javax.crypto.spec.PBEKeySpec;
import javax.inject.Inject;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.spec.InvalidKeySpecException;
import java.util.logging.Level;
import java.util.logging.Logger;

public class PBKDF2EncryptionService implements EncryptionService {
    private static final int ITERATIONS = 1000;
    private static final String PBKDF_2_WITH_HMAC_SHA_1 = "PBKDF2WithHmacSHA1";
    private static final String SEMICOLON = ":";
    private static final String RNG_ALGORITHM = "SHA1PRNG";
    private final HexConverter hexConverter;

    //Bibliography: https://howtodoinjava.com/security/how-to-generate-secure-password-hash-md5-sha-pbkdf2-bcrypt-examples/

    @Inject
    private Logger logger;

    @Inject
    public PBKDF2EncryptionService(HexConverter hexConverter) {
        this.hexConverter = hexConverter;
    }


    @Override
    public boolean validate(String storedPassword, String originalPassword) throws EncryptionException {
        try {
            String[] parts = storedPassword.split(SEMICOLON);
            int iterations = Integer.parseInt(parts[0]);
            byte[] salt = hexConverter.fromHex(parts[1]);
            byte[] validHash = hexConverter.fromHex(parts[2]);
            byte[] hashToValidate = getHash(originalPassword.toCharArray(), salt, iterations, validHash.length * 8);

            return compareByte(validHash, hashToValidate);
        } catch (NoSuchAlgorithmException | InvalidKeySpecException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
            throw new EncryptionException(e);
        }
    }


    public String encrypt(String passwordToEncrypt) throws EncryptionException {
        try {
            return generateStrongPasswordHash(passwordToEncrypt);
        } catch (NoSuchAlgorithmException | InvalidKeySpecException e) {
            logger.log(Level.SEVERE, e.getMessage(), e);
            throw new EncryptionException(e);
        }
    }


    private String generateStrongPasswordHash(String password) throws NoSuchAlgorithmException, InvalidKeySpecException {
        byte[] salt = getSalt();
        byte[] hash = getHash(password.toCharArray(), salt, ITERATIONS, 64 * 8);
        return ITERATIONS + SEMICOLON + hexConverter.toHex(salt) + SEMICOLON + hexConverter.toHex(hash);
    }

    private byte[] getHash(char[] chars, byte[] salt, int iterations, int size) throws NoSuchAlgorithmException, InvalidKeySpecException {
        PBEKeySpec spec = new PBEKeySpec(chars, salt, iterations, size);
        SecretKeyFactory skf = SecretKeyFactory.getInstance(PBKDF_2_WITH_HMAC_SHA_1);
        return skf.generateSecret(spec).getEncoded();
    }

    private byte[] getSalt() throws NoSuchAlgorithmException {
        SecureRandom sr = SecureRandom.getInstance(RNG_ALGORITHM);
        byte[] salt = new byte[16];
        sr.nextBytes(salt);
        return salt;
    }

    private boolean compareByte(byte[] hash, byte[] testHash) {
        int diff = hash.length ^ testHash.length;
        for (int i = 0; i < hash.length && i < testHash.length; i++) {
            diff |= hash[i] ^ testHash[i];
        }
        return diff == 0;
    }


}
