import { Injectable } from '@angular/core';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { Observable, of } from 'rxjs';
import { catchError, map, tap } from 'rxjs/operators';
import { MessageService } from '../message.service';
import { Flow } from 'src/app/services/flows/flow';
import { GenericRestService } from '../generic-rest/generic-rest.service';

declare var window: any;

@Injectable({
  providedIn: 'root'
})
export class FlowService {

  private allFlowsUrl = this.genericRestService.getUrlPrefix() + '/flows';
  private guardTicketFlowsUrl = this.genericRestService.getUrlPrefix() + '/flows?status=GUARDTICKET';
  private workOrderFlowsUrl = this.genericRestService.getUrlPrefix() + '/flows?status=WORKORDER';
  private closedFlowsUrl = this.genericRestService.getUrlPrefix() + '/flows?status=CLOSED';
  private canceledFlowsUrl = this.genericRestService.getUrlPrefix() + '/flows?status=CANCELED';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService: GenericRestService
  ) {

  }

  /** GET flows from the server */
  public getAllFlows(): Observable<Flow[]> {
    return this.getSpecificFlows(this.allFlowsUrl);
  }

  /** GET GuardTicketFlows from the server */
  public getGuardTicketFlows(): Observable<Flow[]> {
    return this.getSpecificFlows(this.guardTicketFlowsUrl);
  }

  /** GET WorkOrderFlows from the server */
  public getWorkOrderFlows(): Observable<Flow[]> {
    return this.getSpecificFlows(this.workOrderFlowsUrl);
  }

  /** GET FinishedFlows from the server */
  public getFinishedFlows(): Observable<Flow[]> {
    return this.getSpecificFlows(this.closedFlowsUrl);
  }

  /** GET CanceledFlows from the server */
  public getCanceledFlows(): Observable<Flow[]> {
    return this.getSpecificFlows(this.canceledFlowsUrl);
  }

  public getFlowsByPlate(plate: string): Observable<Flow[]> {
    return this.http.get<Flow[]>(this.genericRestService.getUrlPrefix() + `/vehicles/${plate}/flows`, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched flows from plate ${plate}`)),
      catchError(this.handleError<Flow[]>('getFlowsByPlate'))
    );
  }

  public getSpecificFlows(url: string): Observable<Flow[]> {
    let finalUrl: string;
    var currentTaller = localStorage.getItem('currentTaller');
    if (currentTaller != null && currentTaller != 'undefined') {
      finalUrl = `${url}&tallerId=${JSON.parse(currentTaller).id}`;
    } else {
      finalUrl = url;
    }
    return this.http.get<Flow[]>(finalUrl, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log('fetched flows')),
      catchError(this.handleError<Flow[]>('getSpecificFlows'))
    );
  }

  public createFlow(flow: Flow): Observable<Flow> {
    return this.http.post<Flow>(this.allFlowsUrl, flow, this.genericRestService.getHeaders()).pipe(
      tap((newFlow: Flow) => this.log(`added flow w/ id=${newFlow.id}`)),
      catchError(this.handleError<Flow>('createFlow')));
  }

  updateFlow(flow: Flow): Observable<Flow> {
    return this.http.put<Flow>(`${this.allFlowsUrl}/${flow.id}`, flow, this.genericRestService.getHeaders()).pipe(
      tap((updatedFlow: Flow) => this.log(`updated flow w/ id=${updatedFlow.id}`)),
      catchError(this.handleError<Flow>('updateFlow')));
  }


  ngOnInit() {
  }

  getFlow(id: number): Observable<Flow> {
    const url = `${this.allFlowsUrl}/${id}`;
    return this.http.get<Flow>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched flow id=${id}`)),
      catchError(this.handleError<Flow>(`getFlow id=${id}`))
    );
  }  

  /**
  * Handle Http operation that failed.
  * Let the app continue.
  * @param operation - name of the operation that failed
  * @param result - optional value to return as the observable result
  */
  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {

      // TODO: send the error to remote logging infrastructure
      console.error(error); // log to console instead

      // TODO: better job of transforming error for user consumption
      this.log(`${operation} failed: ${error.message}`);

      // Let the app keep running by returning an empty result.
      return of(result as T);
    };
  }
  /** Log a HeroService message with the MessageService */
  private log(message: string) {
    this.messageService.add(`HeroService: ${message}`);
  }
}
