import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Product } from './product';
import { HttpClient, HttpHeaders } from '@angular/common/http';
import { MessageService } from '../message.service';
import { catchError, tap } from 'rxjs/operators';
import { GenericRestService } from '../generic-rest/generic-rest.service';


@Injectable({
  providedIn: 'root'
})
export class ProductService {
  private productsURL = this.genericRestService.getUrlPrefix() + '/products';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService: GenericRestService
  ) { }

  getProduct(id: number): Observable<Product> {
    const url = `${this.productsURL}/${id}`;
    return this.http.get<Product>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched product id=${id}`)),
      catchError(this.handleError<Product>(`getProduct id=${id}`))
    );
  }

  getProductByDescription(desc: string): Observable<Product[]> {
    const url = `${this.productsURL}/search/?desc=${desc}`; 
    return this.http.get<Product[]>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched product id=${desc}`)),
      catchError(this.handleError<Product[]>(`getProduct id=${desc}`))
    );
  }

  /** GET clients from the server */
  getProducts(): Observable<Product[]> {
    return this.http.get<Product[]>(this.productsURL, this.genericRestService.getHeaders())
      .pipe(
        tap(_ => this.log('fetched products')),
        catchError(this.handleError<Product[]>('getProducts', []))
      );
  }

  addProduct(product: Product): Observable<Product> {
    return this.http.post<Product>(this.productsURL, product, this.genericRestService.getHeaders()).pipe(
      tap((newProduct: Product) => this.log(`added product w/ id=${newProduct.id}`)),
      catchError(this.handleError<Product>('addProduct')));
  }

  updateProduct(product: Product): Observable<Product> {
    return this.http.put<Product>(`${this.productsURL}/${product.id}`, product, this.genericRestService.getHeaders()).pipe(
      tap((newProduct: Product) => this.log(`updated product w/ id=${newProduct.id}`)),
      catchError(this.handleError<Product>('addProduct')));
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.error(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`ProductService: ${message}`);
  }
}
