import { Injectable } from '@angular/core';
import { Observable, of } from 'rxjs';
import { Vehicle } from './vehicle';
import { HttpClient } from '@angular/common/http';
import { MessageService } from '../message.service';
import { catchError,  tap } from 'rxjs/operators';
import { GenericRestService } from '../generic-rest/generic-rest.service';

@Injectable({
  providedIn: 'root'
})
export class VehicleService {
  private vehiclesURL = this.genericRestService.getUrlPrefix() + '/vehicles';

  constructor(
    private http: HttpClient,
    private messageService: MessageService,
    private genericRestService : GenericRestService
  ) { }

  getVehicle(plate: string): Observable<Vehicle> {
    const url = `${this.vehiclesURL}/${plate}`;
    return this.http.get<Vehicle>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched vehicle id=${plate}`)),
      catchError(this.handleError<Vehicle>(`getVehicle id=${plate}`))
    );
  }

  getVehicles(): Observable<Vehicle[]> {
    return this.http.get<Vehicle[]>(this.vehiclesURL, this.genericRestService.getHeaders())
      .pipe(
        tap(_ => this.log('fetched vehicles')),
        catchError(this.handleError<Vehicle[]>('getVehicles', []))
      );
  }

  getVehiclesByClientId(id: number): Observable<Vehicle[]> {
    const url = `${this.genericRestService.getUrlPrefix()}/clients/${id}/vehicles`;
    return this.http.get<Vehicle[]>(url, this.genericRestService.getHeaders()).pipe(
      tap(_ => this.log(`fetched vehicles from client with id=${id}`)),
      catchError(this.handleError<Vehicle[]>(`getVehiclesByClientId id=${id}`))
    );
  }

  addVehicle(vehicle: Vehicle): Observable<Vehicle> {
    return this.http.post<Vehicle>(this.vehiclesURL, vehicle, this.genericRestService.getHeaders()).pipe(
      tap((newVehicle: Vehicle) => this.log(`added vehicle w/ plate=${newVehicle.plate}`)),
      catchError(this.handleError<Vehicle>('addVehicle')));
  }

  updateVehicle(vehicle: Vehicle): Observable<Vehicle> {
    return this.http.put<Vehicle>(`${this.vehiclesURL}/${vehicle.plate}`, vehicle, this.genericRestService.getHeaders()).pipe(
      tap((updatedVehicle: Vehicle) => this.log(`updated vehicle w/ plate=${updatedVehicle.plate}`)),
      catchError(this.handleError<Vehicle>('updateVehicle')));
  }

  private handleError<T>(operation = 'operation', result?: T) {
    return (error: any): Observable<T> => {
      console.log(error); // log to console instead
      this.log(`${operation} failed: ${error.message}`);
      return of(result as T);
    };
  }

  private log(message: string) {
    this.messageService.add(`VehicleService: ${message}`);
  }
}
